from logging import getLogger
from typing import Optional
from warnings import warn

import aiohttp

from koreanbots.decorator import strict_literal
from koreanbots.errors import KoreanbotsException
from koreanbots.http import KoreanbotsRequester
from koreanbots.model import (
    KoreanbotsBotResponse,
    KoreanbotsResponse,
    KoreanbotsServerResponse,
    KoreanbotsUserResponse,
    KoreanbotsVoteResponse,
)
from koreanbots.typing import VoteType, WidgetStyle, WidgetType

log = getLogger(__name__)


class Koreanbots(KoreanbotsRequester):
    """
    KoreanbotsRequester를 감싸는 클라이언트 클래스입니다.

    :param api_key:
        API key를 지정합니다. 만약 필요한 경우 이 키를 지정하세요.
    :type api_key:
        Optional[str]

    :param session:
        aiohttp.ClientSession의 클래스입니다. 만약 필요한 경우 이 인수를 지정하세요. 지정하지 않으면 생성합니다.
    :type session:
        Optional[aiohttp.ClientSession]
    """

    def __init__(
        self,
        api_key: Optional[str] = None,
        session: Optional[aiohttp.ClientSession] = None,
    ) -> None:
        super().__init__(api_key, session)

    async def post_guild_count(self, bot_id: int, **kwargs: Optional[int]) -> None:
        """
        길드 개수를 서버에 전송합니다.

        :param bot_id:
            요청할 bot의 ID를 지정합니다.
        :type bot_id:
            int
        """
        await super().post_update_bot_info(bot_id, **kwargs)

    async def get_user_info(
        self, user_id: int
    ) -> KoreanbotsResponse[KoreanbotsUserResponse]:
        """
        유저 정보를 가져옵니다.

        :param user_id:
            요청할 유저의 ID를 지정합니다.
        :type user_id:
            int
        :return:
            유저 정보를 담고 있는 KoreanbotsUser클래스입니다.
        :rtype:
            KoreanbotsUser
        """
        data = await super().get_user_info(user_id)

        code = data["code"]
        version = data["version"]
        data = data["data"]

        return KoreanbotsResponse(
            code=code, version=version, data=KoreanbotsUserResponse.from_dict(data)
        )

    async def get_bot_info(
        self, bot_id: int
    ) -> KoreanbotsResponse[KoreanbotsBotResponse]:
        """
        봇 정보를 가져옵니다.

        :param bot_id:
            요청할 봇의 ID를 지정합니다.
        :type bot_id:
            int

        :return:
            봇 정보를 담고 있는 KoreanbotsBot클래스입니다.
        :rtype:
            KoreanbotsBot
        """
        data = await super().get_bot_info(bot_id)

        code = data["code"]
        version = data["version"]
        data = data["data"]

        return KoreanbotsResponse(
            code=code, version=version, data=KoreanbotsBotResponse.from_dict(data)
        )

    async def get_server_info(
        self, server_id: int
    ) -> KoreanbotsResponse[KoreanbotsServerResponse]:
        """
        서버 정보를 가져옵니다.

        :param server_id:
            요청할 서버의 ID를 지정합니다.
        :type server_id:
            int

        :return:
            봇 정보를 담고 있는 KoreanbotsServer클래스입니다.
        :rtype:
            KoreanbotsServer
        """

        data = await super().get_server_info(server_id)

        code = data["code"]
        version = data["version"]
        data = data["data"]

        return KoreanbotsResponse(
            code=code, version=version, data=KoreanbotsServerResponse.from_dict(data)
        )

    @strict_literal(["widget_type", "style"])
    async def get_widget(
        self,
        widget_type: WidgetType,
        bot_id: int,
        style: WidgetStyle = "flat",
        scale: float = 1.0,
        icon: bool = False,
    ) -> str:
        """
        주어진 bot_id로 widget의 url을 반환합니다.

        :param widget_type:
            요청할 widget의 타입을 지정합니다.
        :type widget_type:
            WidgetType

        :param bot_id:
            요청할 bot의 ID를 지정합니다.
        :type bot_id:
            int

        :param style:
            요청할 widget의 형식을 지정합니다. 기본값은 flat로 설정되어 있습니다.
        :type style:
            WidgetStyle, optional

        :param scale:
            요청할 widget의 크기를 지정합니다. 반드시 0.5이상이어야 합니다. 기본값은 1.0입니다.
        :type scale:
            float, optional

        :param icon:
            요청할 widget의 아이콘을 표시할지를 지정합니다. 기본값은 False입니다.
        :type icon:
            bool, optional

        :return:
            위젯 url을 반환합니다.
        :rtype: str
        """
        return await self.get_bot_widget_url(widget_type, bot_id, style, scale, icon)

    async def get_bot_vote(
        self, user_id: int, bot_id: int
    ) -> KoreanbotsResponse[KoreanbotsVoteResponse]:
        """
        user_id를 통해 주어진 bot_id에 대한 투표 여부를 반환합니다.

        :param user_id:
            요청할 user의 ID를 지정합니다.
        :type user_id:
            int

        :param bot_id:
            요청할 봇의 ID를 지정합니다.
        :type bot_id:
            int

        :return:
            투표여부를 담고 있는 KoreanbotsVote클래스입니다.
        :rtype:
            KoreanbotsVote
        """
        data = await super().get_bot_vote(user_id, bot_id)

        code = data["code"]
        version = data["version"]
        data = data["data"]

        return KoreanbotsResponse(
            code=code, version=version, data=KoreanbotsVoteResponse.from_dict(data)
        )

    async def get_server_vote(
        self, user_id: int, server_id: int
    ) -> KoreanbotsResponse[KoreanbotsVoteResponse]:
        """
        user_id를 통해 주어진 server_id에 대한 투표 여부를 반환합니다.

        :param user_id:
            요청할 user의 ID를 지정합니다.
        :type user_id:
            int

        :param server_id:
            요청할 봇의 ID를 지정합니다.
        :type server_id:
            int

        :return:
            투표여부를 담고 있는 KoreanbotsVote클래스입니다.
        :rtype:
            KoreanbotsVote
        """
        data = await super().get_server_vote(user_id, server_id)

        code = data["code"]
        version = data["version"]
        data = data["data"]

        return KoreanbotsResponse(
            code=code, version=version, data=KoreanbotsVoteResponse.from_dict(data)
        )

    # deprecated since 3.0.0

    async def guildcount(self, bot_id: int, **kwargs: Optional[int]) -> None:
        warn(
            "guildcount 메서드는 post_guild_count로 변경되었습니다.", DeprecationWarning
        )

        return await self.post_guild_count(bot_id, **kwargs)

    async def userinfo(
        self, user_id: int
    ) -> KoreanbotsResponse[KoreanbotsUserResponse]:
        warn("userinfo 메서드는 get_user_info로 변경되었습니다.", DeprecationWarning)

        return await self.get_user_info(user_id)

    async def botinfo(self, bot_id: int) -> KoreanbotsResponse[KoreanbotsBotResponse]:
        warn("botinfo 메서드는 get_bot_info로 변경되었습니다.", DeprecationWarning)

        return await self.get_bot_info(bot_id)

    async def serverinfo(
        self, server_id: int
    ) -> KoreanbotsResponse[KoreanbotsServerResponse]:
        warn(
            "serverinfo 메서드는 get_server_info로 변경되었습니다.", DeprecationWarning
        )

        return await self.get_server_info(server_id)

    @strict_literal(["widget_type", "style"])
    async def widget(
        self,
        widget_type: WidgetType,
        bot_id: int,
        style: WidgetStyle = "flat",
        scale: float = 1.0,
        icon: bool = False,
    ) -> str:
        warn("widget 메서드는 get_widget으로 변경되었습니다.", DeprecationWarning)

        return await self.get_widget(widget_type, bot_id, style, scale, icon)

    async def is_voted_bot(
        self, user_id: int, bot_id: int
    ) -> KoreanbotsResponse[KoreanbotsVoteResponse]:
        warn("is_voted_bot 메서드는 get_bot_vote로 변경되었습니다.", DeprecationWarning)

        return await self.get_bot_vote(user_id, bot_id)

    async def is_voted_server(
        self, user_id: int, server_id: int
    ) -> KoreanbotsResponse[KoreanbotsVoteResponse]:
        warn(
            "is_voted_server 메서드는 get_server_vote로 변경되었습니다.",
            DeprecationWarning,
        )

        return await self.get_server_vote(user_id, server_id)



from abc import ABC
from dataclasses import dataclass, field
from typing import Any, Dict, Generic, List, Optional, TypeVar

from .typing import Category, State, Status


class KoreanbotsResponseABC(ABC): ...


T = TypeVar("T", bound=KoreanbotsResponseABC)


@dataclass(frozen=True)
class KoreanbotsResponse(Generic[T]):
    code: int
    """상태 코드"""
    version: int
    """버전"""
    data: T


@dataclass(eq=True, frozen=True)
class KoreanbotsBot(KoreanbotsResponseABC):
    """
    봇의 정보를 가져왔을때 반환되는 인스턴스입니다.
    """

    id: Optional[str] = field(repr=True, compare=True, default=None)
    """아이디"""
    name: Optional[str] = field(repr=True, compare=False, default=None)
    """이름"""
    tag: Optional[str] = field(repr=False, compare=False, default=None)
    """태그"""
    avatar: Optional[str] = field(repr=False, compare=False, default=None)
    """아바타"""
    flags: int = field(repr=False, compare=False, default=0)
    """플래그"""
    lib: Optional[str] = field(repr=False, compare=False, default=None)
    """라이브러리"""
    prefix: Optional[str] = field(repr=False, compare=False, default=None)
    """프리픽스"""
    votes: int = field(repr=False, compare=False, default=0)
    """투표 수"""
    servers: int = field(repr=False, compare=False, default=0)
    """서버 수"""
    shards: int = field(repr=False, compare=False, default=0)
    """샤드 수"""
    intro: Optional[str] = field(repr=False, compare=False, default=None)
    """소개 문구"""
    desc: Optional[str] = field(repr=False, compare=False, default=None)
    """설명 문구"""
    web: Optional[str] = field(repr=False, compare=False, default=None)
    """웹사이트 주소"""
    git: Optional[str] = field(repr=False, compare=False, default=None)
    """깃 주소"""
    url: Optional[str] = field(repr=False, compare=False, default=None)
    """주소"""
    discord: Optional[str] = field(repr=False, compare=False, default=None)
    """디스코드 주소"""
    category: Optional[Category] = field(repr=False, compare=False, default=None)
    """카테고리"""
    vanity: Optional[str] = field(repr=False, compare=False, default=None)
    """가상 주소"""
    bg: Optional[str] = field(repr=False, compare=False, default=None)
    """배경 이미지 주소"""
    banner: Optional[str] = field(repr=False, compare=False, default=None)
    """배너 이미지 주소"""
    status: Optional[Status] = field(repr=False, compare=False, default=None)
    """상태"""
    state: Optional[State] = field(repr=False, compare=False, default=None)
    """Koreanbots에서의 상태"""


@dataclass(eq=True, frozen=True)
class KoreanbotsUser(KoreanbotsResponseABC):
    """
    유저 정보를 가져왔을때 반환되는 클래스입니다.
    """

    id: int = field(repr=True, compare=True, default=0)
    """아이디"""
    username: str = field(repr=True, compare=False, default="")
    """유저 이름"""
    globalName: str = field(repr=True, compare=False, default="")
    """유저 글로벌 이름"""
    tag: str = field(repr=False, compare=False, default="")
    """태그"""
    github: Optional[str] = field(repr=False, compare=False, default=None)
    """Github 주소"""
    flags: int = field(repr=False, compare=False, default=0)
    """플래그"""


@dataclass(eq=True, frozen=True)
class KoreanbotsServer(KoreanbotsResponseABC):
    """
    서버 정보를 가져왔을때 반환되는 클래스입니다.
    """

    id: int = field(repr=True, compare=True, default=0)
    """ID"""
    name: str = field(repr=True, compare=False, default="")
    """서버 이름"""
    flags: int = field(repr=False, compare=False, default=0)
    """플래그"""
    intro: Optional[str] = field(repr=False, compare=False, default=None)
    """소개문구"""
    desc: Optional[str] = field(repr=False, compare=False, default=None)
    """설명문구"""
    votes: int = field(repr=True, compare=False, default=0)
    """투표수"""
    category: Optional[Category] = field(repr=False, compare=False, default=None)
    """카테고리"""
    invite: str = field(repr=False, compare=False, default="")
    """초대링크"""
    state: Optional[State] = field(repr=False, compare=False, default=None)
    """Koreanbots에서의 상태"""
    vanity: Optional[str] = field(repr=False, compare=False, default=None)
    """서버의 가상 주소"""
    bg: Optional[str] = field(repr=False, compare=False, default=None)
    """배경 이미지 주소"""
    banner: Optional[str] = field(repr=False, compare=False, default=None)
    """배너 이미지 주소"""
    icon: Optional[str] = field(repr=False, compare=False, default=None)
    """아이콘"""
    members: int = field(repr=False, compare=False, default=0)
    """멤버 수"""
    emojis: List["Emoji"] = field(repr=False, compare=False, default_factory=list)
    """Emoji 인스턴스를 담고 있는 리스트"""
    boostTier: int = field(repr=False, compare=False, default=0)
    """부스트 레벨"""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "KoreanbotsServer":
        return cls(
            id=data["id"],
            name=data["name"],
            flags=data["flags"],
            intro=data.get("intro"),
            desc=data.get("desc"),
            votes=data["votes"],
            category=data.get("category"),
            invite=data["invite"],
            state=data.get("state"),
            vanity=data.get("vanity"),
            bg=data.get("bg"),
            banner=data.get("banner"),
            icon=data.get("icon"),
            members=data["members"],
            emojis=[
                Emoji(id=e["id"], name=e["name"], url=e["url"]) for e in data["emojis"]
            ],
            boostTier=data["boostTier"],
        )


@dataclass(eq=True, frozen=True)
class CircularKoreanbotsBot(KoreanbotsBot):
    owners: List[str] = field(repr=False, compare=False, default_factory=list)


@dataclass(eq=True, frozen=True)
class CircularKoreanbotsUser(KoreanbotsUser):
    bots: List[str] = field(repr=False, compare=False, default_factory=list)
    servers: List[str] = field(repr=False, compare=False, default_factory=list)


@dataclass(eq=True, frozen=True)
class CircularKoreanbotsServer(KoreanbotsServer):
    owner: str = field(repr=False, compare=False, default="")


@dataclass(eq=True, frozen=True)
class KoreanbotsUserResponse(KoreanbotsUser):
    bots: List["CircularKoreanbotsBot"] = field(
        repr=False, compare=False, default_factory=list
    )
    servers: List[CircularKoreanbotsServer] = field(
        repr=False, compare=False, default_factory=list
    )

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "KoreanbotsUserResponse":
        return cls(
            id=data["id"],
            username=data["username"],
            globalName=data["globalName"],
            tag=data["tag"],
            github=data.get("github"),
            flags=data["flags"],
            servers=[
                CircularKoreanbotsServer(
                    id=s["id"],
                    name=s["name"],
                    flags=s["flags"],
                    intro=s.get("intro"),
                    desc=s.get("desc"),
                    votes=s["votes"],
                    category=s.get("category"),
                    invite=s["invite"],
                    state=s.get("state"),
                    vanity=s.get("vanity"),
                    bg=s.get("bg"),
                    banner=s.get("banner"),
                    icon=s.get("icon"),
                    members=s["members"],
                    emojis=[
                        Emoji(id=e["id"], name=e["name"], url=e["url"])
                        for e in s["emojis"]
                    ],
                    boostTier=s["boostTier"],
                    owner=s["owner"],
                )
                for s in data["servers"]
            ],
            bots=[
                CircularKoreanbotsBot(
                    id=b["id"],
                    name=b["name"],
                    tag=b["tag"],
                    avatar=b["avatar"],
                    flags=b["flags"],
                    lib=b["lib"],
                    prefix=b["prefix"],
                    votes=b["votes"],
                    servers=b["servers"],
                    shards=b["shards"],
                    intro=b.get("intro"),
                    desc=b.get("desc"),
                    web=b.get("web"),
                    git=b.get("git"),
                    url=b.get("url"),
                    discord=b.get("discord"),
                    category=b.get("category"),
                    vanity=b.get("vanity"),
                    bg=b.get("bg"),
                    banner=b.get("banner"),
                    status=b.get("status"),
                    state=b.get("state"),
                    owners=b["owners"],
                )
                for b in data["bots"]
            ],
        )


@dataclass(eq=True, frozen=True)
class KoreanbotsBotResponse(KoreanbotsBot):
    owners: List["CircularKoreanbotsUser"] = field(
        repr=False, compare=False, default_factory=list
    )

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "KoreanbotsBotResponse":
        return cls(
            id=data["id"],
            name=data["name"],
            tag=data["tag"],
            avatar=data["avatar"],
            flags=data["flags"],
            lib=data["lib"],
            prefix=data["prefix"],
            votes=data["votes"],
            servers=data["servers"],
            shards=data["shards"],
            intro=data.get("intro"),
            desc=data.get("desc"),
            web=data.get("web"),
            git=data.get("git"),
            url=data.get("url"),
            discord=data.get("discord"),
            category=data.get("category"),
            vanity=data.get("vanity"),
            bg=data.get("bg"),
            banner=data.get("banner"),
            status=data.get("status"),
            state=data.get("state"),
            owners=[
                CircularKoreanbotsUser(
                    id=o["id"],
                    username=o["username"],
                    globalName=o["globalName"],
                    tag=o["tag"],
                    github=o.get("github"),
                    flags=o["flags"],
                    bots=o["bots"],
                )
                for o in data["owners"]
            ],
        )


@dataclass(eq=True, frozen=True)
class KoreanbotsServerResponse(KoreanbotsServer):
    owner: "CircularKoreanbotsUser" = field(
        repr=False, compare=False, default=CircularKoreanbotsUser()
    )

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "KoreanbotsServerResponse":
        return cls(
            id=data["id"],
            name=data["name"],
            flags=data["flags"],
            intro=data.get("intro"),
            desc=data.get("desc"),
            votes=data["votes"],
            category=data.get("category"),
            invite=data["invite"],
            state=data.get("state"),
            vanity=data.get("vanity"),
            bg=data.get("bg"),
            banner=data.get("banner"),
            icon=data.get("icon"),
            members=data["members"],
            emojis=[
                Emoji(id=e["id"], name=e["name"], url=e["url"]) for e in data["emojis"]
            ],
            boostTier=data["boostTier"],
            owner=CircularKoreanbotsUser(
                id=data["owner"]["id"],
                username=data["owner"]["username"],
                globalName=data["owner"]["globalName"],
                tag=data["owner"]["tag"],
                github=data["owner"].get("github"),
                flags=data["owner"]["flags"],
                bots=data["owner"]["bots"],
                servers=data["owner"]["servers"],
            ),
        )


@dataclass(eq=True, frozen=True)
class Emoji:
    """
    이모지 정보를 가져왔을때 반환되는 클래스입니다.
    """

    id: int = field(repr=True, compare=True, default=0)
    """ID"""
    name: str = field(repr=True, compare=False, default="")
    """이모지 이름"""
    url: str = field(repr=False, compare=False, default="")
    """이모지 url"""


@dataclass(eq=True, frozen=True)
class KoreanbotsVoteResponse(KoreanbotsResponseABC):
    voted: bool = field(repr=True, compare=True, default=False)
    """투표 여부"""
    last_vote: int = field(repr=True, compare=True, default=0)
    """마지막으로 투표한 일자"""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "KoreanbotsVoteResponse":
        return cls(voted=data["voted"], last_vote=data["lastVote"])



import pytest

from koreanbots.client import Koreanbots
from koreanbots.model import KoreanbotsBot, KoreanbotsUser


@pytest.mark.asyncio
async def test_botinfo(session: Koreanbots):
    response = await session.get_bot_info(653534001742741552)
    assert response.code == 200
    assert response.data.name == "KODL"
    assert isinstance(response.data.owners[0], KoreanbotsUser)
    assert isinstance(response.data.owners[0].bots[0], str)


@pytest.mark.asyncio
async def test_get_user_info(session: Koreanbots):
    response = await session.get_user_info(285185716240252929)

    assert response.code == 200
    assert isinstance(response.data.bots[0], KoreanbotsBot)
    assert isinstance(response.data.bots[0].owners[0], str)


@pytest.mark.asyncio
async def test_get_server_info(session: Koreanbots):
    response = await session.get_server_info(653083797763522580)

    assert response.code == 200
    assert isinstance(response.data.owner, KoreanbotsUser)


@pytest.mark.asyncio
async def test_get(session: Koreanbots):
    response = await session.get_widget("votes", 653534001742741552)
    assert (
        response
        == "https://koreanbots.dev/api/v2/widget/bots/votes/653534001742741552.svg?style=flat&scale=1.0&icon=False"
    )



import logging
from asyncio.events import get_event_loop
from asyncio.tasks import sleep
from typing import Optional

from aiohttp import ClientSession

try:
    from dico import Client  # type: ignore
except ImportError:
    pass

from koreanbots.client import Koreanbots

log = logging.getLogger(__name__)


class DicoKoreanbots(Koreanbots):
    """
    KoreanbotsRequester를 감싸는 클라이언트 클래스입니다.
    dico 전용입니다.

    :param client:
        dico.Client의 클래스입니다.
    :type client:
        dico.Client

    :param api_key:
        API key를 지정합니다.
    :type api_key:
        str

    :param session:
        aiohttp.ClientSession의 클래스입니다. 만약 필요한 경우 이 인수를 지정하세요. 지정하지 않으면 생성합니다.
    :type session:
        Optional[aiohttp.ClientSession]

    :param run_task:
        봇 정보를 갱신하는 작업을 자동으로 실행합니다. 만약 아니라면 지정하지 않습니다.
    :type run_task:
        bool

    :param include_shard_count:
        샤드 개수를 포함할지 지정합니다. 만약 아니라면 지정하지 않습니다.
    :type include_shard_count:
        bool
    """

    def __init__(
        self,
        client: "Client",
        api_key: str,
        session: Optional[ClientSession] = None,
        run_task: bool = False,
        include_shard_count: bool = False,
    ):
        self.client = client

        if client:
            original_close = getattr(client, "close")

            async def close() -> None:
                if self.session is not None and not self.session.closed:
                    await self.session.close()
                await original_close()

            setattr(client, "close", close)

        self.include_shard_count = include_shard_count
        super().__init__(api_key, session)

        if run_task:
            get_event_loop().create_task(self.tasks_send_guildcount())

    async def tasks_send_guildcount(self) -> None:
        """
        길드 개수를 서버에 전송하는 태스크 입니다.

        :raises RuntimeError:
            클라이언트를 찾을 수 없습니다.
        """

        if not self.client:
            raise RuntimeError("Client Not Found")

        await self.client.wait_ready()

        while not self.client.websocket_closed:
            if not self.client.application_id:
                continue

            kwargs = {"servers": self.client.guild_count}
            if self.include_shard_count:
                if self.client.shard_count:
                    kwargs.update({"shards": self.client.shard_count})
            log.info("Initiating guild count update...")
            try:
                await self.post_guild_count(int(self.client.application_id), **kwargs)
            except:
                log.exception("Guild count update failed due to an error.")
            else:
                log.info(
                    "Guild count updated successfully. Waiting 30 minutes for the next update."
                )
            await sleep(1800)



from asyncio.tasks import Task, sleep
from logging import getLogger
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Coroutine,
    Optional,
    TypeVar,
    Union,
    cast,
)

from aiohttp import ClientSession

from koreanbots.client import Koreanbots

if TYPE_CHECKING:
    import nextcord
    from discord import Client as DiscordpyClient
    from disnake.client import Client as DisnakeClient

T = TypeVar("T")
Coro = Coroutine[Any, Any, T]
CoroT = TypeVar("CoroT", bound=Callable[..., Coro[Any]])

log = getLogger(__name__)


class DiscordpyKoreanbots(Koreanbots):
    """
    Koreanbots를 감싸는 클라이언트 클래스입니다.
    discord.py 및 해당 라이브러리의 포크 전용입니다.

    :param client:
        discord.Client의 클래스입니다. 만약 필요한 경우 이 인수를 지정하세요.
    :type client:
        Optional[DpyABC]

    :param api_key:
        API key를 지정합니다. 만약 필요한 경우 이 키를 지정하세요.
    :type api_key:
        Optional[str]

    :param session:
        aiohttp.ClientSession의 클래스입니다. 만약 필요한 경우 이 인수를 지정하세요. 지정하지 않으면 생성합니다.
    :type session:
        Optional[aiohttp.ClientSession]

    :param run_task:
        봇 정보를 갱신하는 작업을 자동으로 실행합니다. 만약 아니라면 지정하지 않습니다.
    :type run_task:
        bool

    :param include_shard_count:
        샤드 개수를 포함할지 지정합니다. 만약 아니라면 지정하지 않습니다.
    :type include_shard_count:
        bool
    """

    def __init__(
        self,
        client: Union["DiscordpyClient", "nextcord.Client", "DisnakeClient"],
        api_key: str,
        session: Optional[ClientSession] = None,
        run_task: bool = False,
        include_shard_count: bool = False,
    ):
        self.client = client

        # Patch discord.py client.close() method to handle session.close()
        original_close = getattr(client, "close")

        async def close() -> None:
            if self.session is not None and not self.session.closed:
                await self.session.close()
            await original_close()

        setattr(client, "close", close)

        self.include_shard_count = include_shard_count
        super().__init__(api_key, session)

        if run_task:
            client_ready = getattr(client, "on_ready", None)
            client_event = getattr(client, "event")
            self.guildcount_sender: Optional[Task[None]] = None

            # Set default on_ready handler to start send_guildcount task.
            if client_ready is not None:

                async def on_ready() -> None:
                    self.run_post_guild_count_task()
                    await client_ready()  # call previously registered on_ready handler.

            else:

                async def on_ready() -> None:
                    self.run_post_guild_count_task()

            def event(coro: CoroT, /) -> CoroT:
                if coro.__name__ == "on_ready":

                    async def on_ready() -> None:
                        self.run_post_guild_count_task()
                        await coro()

                    return cast(CoroT, client_event(on_ready))

                return cast(CoroT, client_event(coro))

            client.event(on_ready)
            setattr(client, "event", event)

    @property
    def is_running(self) -> bool:
        return self.guildcount_sender is not None and not self.guildcount_sender.done()

    def run_post_guild_count_task(self) -> None:
        """
        tasks_send_guildcount를 호출하는 함수입니다.
        사용자가 on_ready 이벤트 핸들러를 정의할 때, 이 함수를 호출해 길드 개수를 지속적으로 갱신할 수 있습니다.
        """
        if not self.is_running:
            self.guildcount_sender = self.client.loop.create_task(
                self.tasks_send_guildcount()
            )

    async def tasks_send_guildcount(self) -> None:
        """
        길드 개수를 서버에 전송하는 태스크 입니다.

        :raises RuntimeError:
            클라이언트를 찾을 수 없습니다.
        """

        if not self.client:
            raise RuntimeError("Client Not Found")

        await self.client.wait_until_ready()

        while not self.client.is_closed():
            if not self.client.user:
                continue

            kwargs = {"servers": len(self.client.guilds)}
            if self.include_shard_count:
                if self.client.shard_count:
                    kwargs.update({"shards": self.client.shard_count})
            log.info("Initiating guild count update...")
            try:
                await self.post_guild_count(int(self.client.user.id), **kwargs)
            except:
                log.exception("Guild count update failed due to an error.")
            else:
                log.info(
                    "Guild count updated successfully. Waiting 30 minutes for the next update."
                )
            await sleep(1800)



import functools
import inspect
from typing import Any, Callable, List, Literal, cast, get_args

from koreanbots.typing import CORO


def strict_literal(argument_names: List[str]) -> Callable[[CORO], CORO]:
    def decorator(f: CORO) -> CORO:
        @functools.wraps(f)
        async def decorated_function(*args: Any, **kwargs: Any) -> Any:
            # First get about func args
            full_arg_spec = inspect.getfullargspec(f)
            for argument_name in argument_names:
                # Get annotation
                arg_annoration = full_arg_spec.annotations[argument_name]
                # Check annotation is Lireral
                if arg_annoration.__origin__ is Literal:
                    # Literal -> list
                    literal_list = list(get_args(arg_annoration))
                    # Get index
                    arg_index = full_arg_spec.args.index(argument_name)
                    # Handle arguments
                    if arg_index < len(args) and args[arg_index] not in literal_list:
                        raise ValueError(
                            f"Arguments do not match. Expected: {literal_list}"
                        )
                    # Handle keyword arguments
                    elif (
                        kwargs.get(argument_name)
                        and kwargs[argument_name] not in literal_list
                    ):
                        if kwargs[argument_name] not in literal_list:
                            raise ValueError(
                                f"Arguments do not match. Expected: {literal_list}"
                            )

            return await f(*args, **kwargs)

        return cast(CORO, decorated_function)

    return decorator



from typing import Literal, NamedTuple

from .client import Koreanbots as Koreanbots
from .errors import *
from .http import KoreanbotsRequester as KoreanbotsRequester
from .model import KoreanbotsBot as KoreanbotsBot
from .model import KoreanbotsServer as KoreanbotsServer
from .model import KoreanbotsUser as KoreanbotsUser


class VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: Literal["alpha", "beta", "candidate", "final"]
    serial: int


versioninfo = VersionInfo(3, 0, 0, "final", 0)

__version__ = "3.0.0"
